@@ -1,4 +1,3 @@
-# Scientific libraries
 import numpy as np
 import scipy.stats as stats
 import scipy.special as sf
@@ -7,18 +6,21 @@ import pandas as pd
 import abc
 from IPython.display import display, Math, Markdown
 
+import networkx as nx
+
+
 from popsynth.population import Population
 from popsynth.auxiliary_sampler import DerivedLumAuxSampler
 from popsynth.utils.rejection_sample import rejection_sample
 from popsynth.distribution import LuminosityDistribution, SpatialDistribution
 
+
 # from popsynth.utils.progress_bar import progress_bar
 from tqdm.autonotebook import tqdm as progress_bar
 from numba import jit, njit, prange, float64
 
 
-class PopulationSynth(object):
-    __metaclass__ = abc.ABCMeta
+class PopulationSynth(object, metaclass=abc.ABCMeta):
 
     def __init__(
         self,
@@ -42,6 +44,8 @@ class PopulationSynth(object):
         self._model_spaces = {}
         self._auxiliary_observations = {}
 
+        self._graph = nx.DiGraph()
+
         self._verbose = verbose
 
         self._name = "%s" % spatial_distribution.name
@@ -74,6 +78,12 @@ class PopulationSynth(object):
 
                 self._params[k] = v
 
+        self._graph.add_node(self._spatial_distribution.name)
+
+        # add the sky sampler
+
+        
+        
     @property
     def spatial_distribution(self):
         return self._spatial_distribution
@@ -122,6 +132,8 @@ class PopulationSynth(object):
 
         else:
 
+            assert not auxiliary_sampler.is_secondary, f'{auxiliary_sampler.name} is already set as a secondary sampler!'
+            assert auxiliary_sampler.name not in self._auxiliary_observations, f'{auxiliary_sampler.name} is already registered!'
             if self._verbose:
                 print("registering auxilary sampler: %s" % auxiliary_sampler.name)
 
@@ -195,17 +207,23 @@ class PopulationSynth(object):
 
         # integrate the population to determine the true number of
         # objects
-        N = integrate.quad(dNdr, 0.0, self._spatial_distribution._r_max)[0]
+        N = integrate.quad(dNdr, 0.0, self._spatial_distribution.r_max)[0]
 
         if verbose:
             print("The volume integral is %f" % N)
 
         # this should be poisson distributed
         n = np.random.poisson(N)
-        #       pbar.update()
-        #       pbar.set_description(desc='Drawing distances')
-        distances = self._spatial_distribution.draw_distance(size=n, verbose=verbose)
 
+        
+        self._spatial_distribution.draw_distance(size=n, verbose=verbose)
+
+        # now draw the sky positions
+        
+        self._spatial_distribution.draw_sky_positions(size=n)
+
+        distances = self._spatial_distribution.distances
+        
         if verbose:
             print("Expecting %d total objects" % n)
 
@@ -254,7 +272,6 @@ class PopulationSynth(object):
             auxiliary_quantities[self._derived_luminosity_sampler.name] = {
                 "true_values": self._derived_luminosity_sampler.true_values,
                 "obs_values": self._derived_luminosity_sampler.obs_values,
-                "sigma": self._derived_luminosity_sampler.sigma,
                 "selection": self._derived_luminosity_sampler.selection,
             }
             if verbose:
@@ -273,7 +290,7 @@ class PopulationSynth(object):
 
                 # first we tell the sampler to go and retrieve all of
                 # its own secondaries
-
+                
                 properties = v2.get_secondary_properties()
 
                 for k3, v3 in properties.items():
@@ -335,7 +352,6 @@ class PopulationSynth(object):
             # append these values to a dict
             auxiliary_quantities[k] = {
                 "true_values": v.true_values,
-                "sigma": v.sigma,
                 "obs_values": v.obs_values,
                 "selection": v.selection,
             }
@@ -364,6 +380,8 @@ class PopulationSynth(object):
         # this is homoskedastic for now
         log10_fluxes_obs = self.draw_log10_fobs(fluxes, flux_sigma, size=n)
 
+        assert np.alltrue(np.isfinite(log10_fluxes_obs))
+        
         # now select them
 
         if not hard_cut:
@@ -385,6 +403,7 @@ class PopulationSynth(object):
                 ):
 
                     # make a bernoulli draw given the detection probability
+
                     if stats.bernoulli.rvs(p) == 1:
 
                         selection.append(True)
@@ -442,13 +461,14 @@ class PopulationSynth(object):
                 )
 
         selection = np.logical_and(selection, auxiliary_selection)
+
+        # if we do not want to add a selection effect
         if no_selection:
             if self._verbose:
                 print("No Selection! Added back all objects")
 
             selection = np.ones_like(selection, dtype=bool)
 
-        
         # pbar.update()
         if sum(selection) == n:
 
@@ -533,7 +553,6 @@ class PopulationSynth(object):
         if distance_probability is None:
             distance_probability = 1.0
 
-            
         return Population(
             luminosities=luminosities,
             distances=distances,
@@ -559,6 +578,9 @@ class PopulationSynth(object):
             truth=truth,
             hard_cut=hard_cut,
             distance_probability=distance_probability,
+            graph=self.graph,
+            theta = self._spatial_distribution.theta,
+            phi = self._spatial_distribution.phi
         )
 
     def display(self):
@@ -594,3 +616,92 @@ class PopulationSynth(object):
     # def generate_stan_code(self, stan_gen, **kwargs):
 
     #     pass
+
+    @property
+    def graph(self):
+
+        self._build_graph()
+
+        return self._graph
+
+    def _build_graph(self):
+        """
+        builds the graph for all the samplers
+
+        :returns: 
+        :rtype: 
+
+        """
+
+        # first check out the luminosity sampler
+
+        self._graph.add_node("obs_flux", observed=True)
+        self._graph.add_edge(self._spatial_distribution.name, "obs_flux")
+        if self._has_derived_luminosity:
+
+            self._graph.add_node(self._derived_luminosity_sampler.name)
+
+            self._graph.add_edge(self._derived_luminosity_sampler.name, "obs_flux")
+
+            if self._derived_luminosity_sampler.uses_distance:
+
+                self._graph.add_edge(
+                    self._spatial_distribution.name,
+                    self._derived_luminosity_sampler.name,
+                )
+
+            for k2, v2 in self._derived_luminosity_sampler.secondary_samplers.items():
+
+                self._graph.add_node(k2)
+                self._graph.add_edge(k2, self._derived_luminosity_sampler.name)
+
+                # pass the graph and the primary
+
+                _ = v2.get_secondary_properties(
+                    graph=self._graph,
+                    primary=k2,
+                    spatial_distribution=self._spatial_distribution,
+                )
+
+        else:
+            self._graph.add_edge(self._luminosity_distribution.name, "obs_flux")
+        # now do the same fro everything else
+
+        for k, v in self._auxiliary_observations.items():
+
+            assert (
+                not v.is_secondary
+            ), "This is a secondary sampler. You cannot sample it in the main sampler"
+
+            self._graph.add_node(k, observed=False)
+
+            if v.observed:
+                self._graph.add_node(v.obs_name, observed=False)
+                self._graph.add_edge(k, v.obs_name)
+
+                if v.uses_distance:
+
+                    self._graph.add_edge(self._spatial_distribution.name, k)
+
+            for k2, v2 in v.secondary_samplers.items():
+
+                # first we tell the sampler to go and retrieve all of
+                # its own secondaries
+
+                self._graph.add_edge(k2, k)
+                self._graph.add_node(k2, observed=False)
+
+                if v2.uses_distance:
+
+                    self._graph.add_edge(self._spatial_distribution.name, k2)
+
+                if v2.observed:
+
+                    self._graph.add_node(v2.obs_name, observed=True)
+                    self._graph.add_edge(k2, v2.obs_name)
+
+                _ = v2.get_secondary_properties(
+                    graph=self._graph,
+                    primary=k2,
+                    spatial_distribution=self._spatial_distribution,
+                )
